import { Component, Host, h, Prop, State, Watch, Element } from '@stencil/core';
import { defaultConfig } from '../../lib/default-ui-config';
import { defaultIconPack } from '../../lib/icons';
import { useLanguage } from '../../lib/lang';
import { PeersGrid } from './components/PeersGrid';
import { Render } from '../../lib/render';
export class DyteGrid {
  constructor() {
    /** The aspect ratio of each peer */
    this.aspectRatio = '16:9';
    /** Gap between peers */
    this.gap = 8;
    /** Config object */
    this.config = defaultConfig;
    /** Icon pack */
    this.iconPack = defaultIconPack;
    /** Language */
    this.t = useLanguage();
    this.peers = [];
    this.screensharePeers = [];
    this.plugins = [];
  }
  connectedCallback() {
    this.resizeObserver = new ResizeObserver(() => {
      // update parent dimensions
      const { clientWidth, clientHeight } = this.host;
      this.dimensions = [clientWidth, clientHeight];
    });
    this.resizeObserver.observe(this.host);
    this.meetingChanged(this.meeting);
  }
  disconnectedCallback() {
    var _a;
    (_a = this.resizeObserver) === null || _a === void 0 ? void 0 : _a.disconnect();
    if (this.meeting == null)
      return;
    this.participantJoinedListener &&
      this.meeting.participants.active.removeListener('participantJoined', this.participantJoinedListener);
    this.participantJoinedListener &&
      this.meeting.participants.active.removeListener('participantLeft', this.participantJoinedListener);
    this.screenShareListener &&
      this.meeting.participants.joined.removeListener('screenShareUpdate', this.screenShareListener);
    if (this.viewModeListener != null) {
      this.meeting.participants.removeListener('viewModeChanged', this.viewModeListener);
      this.meeting.participants.removeListener('pageChanged', this.viewModeListener);
    }
    this.screenShareListener &&
      this.meeting.self.removeListener('screenShareUpdate', this.screenShareListener);
  }
  meetingChanged(meeting) {
    if (meeting != null) {
      this.peers = [
        ...meeting.participants.active.toArray(),
        meeting.participants.viewMode === 'ACTIVE_GRID' && meeting.self,
      ];
      this.plugins = [...meeting.plugins.active.toArray()];
      if (this.participantJoinedListener != null)
        return;
      this.participantJoinedListener = (peer) => {
        if (!this.peers.some((p) => p.id === peer.id)) {
          this.peers = [peer, ...this.peers];
        }
      };
      this.participantLeftListener = (peer) => {
        this.peers = this.peers.filter((p) => p.id !== peer.id);
        this.screensharePeers = this.screensharePeers.filter((p) => p.id !== peer.id);
      };
      this.screenShareListener = (peer) => {
        if (peer.screenShareEnabled && peer.screenShareTracks.video != null) {
          this.addScreenSharePeer(peer);
        }
        else {
          this.removeScreenSharePeer(peer.id);
        }
      };
      this.selfScreenShareListener = ({ screenShareEnabled, screenShareTracks }) => {
        const peer = meeting.self;
        if (screenShareEnabled && screenShareTracks.video != null) {
          this.addScreenSharePeer(peer);
        }
        else {
          this.removeScreenSharePeer(peer.id);
        }
      };
      this.viewModeListener = ({ viewMode, currentPage, pageCount }) => {
        if (viewMode === 'ACTIVE_GRID') {
          if (!this.peers.some((p) => p.id === meeting.self.id)) {
            // add self if it doesn't exist
            this.peers = [...this.peers, this.meeting.self];
          }
        }
        else if (currentPage !== pageCount) {
          // remove self peer only if the current page is not the last page
          this.peers = this.peers.filter((p) => p.id !== this.meeting.self.id);
        }
      };
      this.pageChangedListener = ({ currentPage, pageCount, viewMode }) => {
        if (viewMode === 'PAGINATED') {
          if (currentPage === pageCount) {
            if (!this.peers.some((p) => p.id === meeting.self.id)) {
              // add self peer if it is the last page and it already isn't added
              this.peers = [...this.peers, meeting.self];
            }
          }
        }
      };
      this.pluginUpdateListener = (plugin, { active }) => {
        if (active) {
          if (!this.plugins.some((p) => p.id === plugin.id)) {
            this.plugins = [...this.plugins, plugin];
          }
        }
        else {
          this.plugins = this.plugins.filter((p) => p.id !== plugin.id);
        }
      };
      {
        const activeParticipants = meeting.participants.active.toArray();
        for (const peer of activeParticipants) {
          this.screenShareListener(peer);
        }
      }
      meeting.participants.active.addListener('participantJoined', this.participantJoinedListener);
      meeting.participants.active.addListener('participantLeft', this.participantLeftListener);
      meeting.participants.active.addListener('screenShareUpdate', this.screenShareListener);
      meeting.participants.addListener('viewModeChanged', this.viewModeListener);
      meeting.participants.addListener('pageChanged', this.pageChangedListener);
      meeting.self.addListener('screenShareUpdate', this.selfScreenShareListener);
      meeting.plugins.all.addListener('stateUpdate', this.pluginUpdateListener);
    }
  }
  addScreenSharePeer(peer) {
    if (!this.screensharePeers.some((p) => p.id === peer.id)) {
      this.screensharePeers = [...this.screensharePeers, peer];
    }
  }
  removeScreenSharePeer(id) {
    this.screensharePeers = this.screensharePeers.filter((p) => p.id !== id);
  }
  render() {
    const isPeersOnly = this.screensharePeers.length + this.plugins.length === 0;
    const defaults = {
      meeting: this.meeting,
      size: this.size,
      states: this.states,
      config: this.config,
      iconPack: this.iconPack,
      t: this.t,
    };
    const common = {
      peers: this.peers,
      defaults,
      gap: this.gap,
      aspectRatio: this.aspectRatio,
      containerDimensions: this.dimensions,
      host: this.host,
    };
    return (h(Host, null,
      isPeersOnly && (h(PeersGrid, Object.assign({}, common, { host: this.host, containerDimensions: this.dimensions }))),
      !isPeersOnly && (h("dyte-full-grid", Object.assign({}, defaults, common, { screenSharePeers: this.screensharePeers, plugins: this.plugins }))),
      h(Render, { element: "dyte-grid", defaults: defaults, onlyChildren: true })));
  }
  static get is() { return "dyte-grid"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["dyte-grid.css"]
  }; }
  static get styleUrls() { return {
    "$": ["dyte-grid.css"]
  }; }
  static get properties() { return {
    "aspectRatio": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The aspect ratio of each peer"
      },
      "attribute": "aspect-ratio",
      "reflect": true,
      "defaultValue": "'16:9'"
    },
    "meeting": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "Meeting",
        "resolved": "DyteClient",
        "references": {
          "Meeting": {
            "location": "import",
            "path": "../../types/dyte-client"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Meeting object"
      }
    },
    "gap": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Gap between peers"
      },
      "attribute": "gap",
      "reflect": true,
      "defaultValue": "8"
    },
    "size": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "Size",
        "resolved": "\"lg\" | \"md\" | \"sm\" | \"xl\"",
        "references": {
          "Size": {
            "location": "import",
            "path": "../../types/props"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Size"
      },
      "attribute": "size",
      "reflect": true
    },
    "states": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "States",
        "resolved": "States",
        "references": {
          "States": {
            "location": "import",
            "path": "../../types/props"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "States"
      }
    },
    "config": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "UIConfig",
        "resolved": "UIConfig",
        "references": {
          "UIConfig": {
            "location": "import",
            "path": "../../types/ui-config"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Config object"
      },
      "defaultValue": "defaultConfig"
    },
    "iconPack": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "IconPack",
        "resolved": "{ live: string; people: string; chat: string; poll: string; participants: string; rocket: string; video_background_effect: string; call_end: string; grid: string; share: string; check_mark: string; mic_on: string; mic_off: string; video_on: string; video_off: string; share_screen_start: string; share_screen_stop: string; share_screen_person: string; speaker_off: string; avatar: string; clock: string; dismiss: string; send: string; search: string; more_vertical: string; chevron_left: string; chevron_right: string; settings: string; wifi: string; speaker: string; download: string; arrow_expand: string; copy: string; attach: string; image: string; emoji_multiple: string; vertical_scroll: string; arrow_minimize: string; image_off: string; disconnected: string; poor_network: string; wand: string; rejoin: string; }",
        "references": {
          "IconPack": {
            "location": "import",
            "path": "../../lib/icons"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Icon pack"
      },
      "defaultValue": "defaultIconPack"
    },
    "t": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "(key: \"audio\" | \"video\" | \"chat\" | \"participants\" | \"polls\" | \"plugins\" | \"mic_off\" | \"mic_on\" | \"video_off\" | \"video_on\" | \"left_meeting\" | \"share_screen_start\" | \"share_screen_stop\" | \"leave\" | \"dismiss\" | \"page\" | \"settings\" | \"connection\" | \"leave_confirmation\" | \"end_meeting_for_all\" | \"cancel\" | (string & {})) => string",
        "resolved": "(key: \"audio\" | \"video\" | \"chat\" | \"participants\" | \"polls\" | \"plugins\" | \"mic_off\" | \"mic_on\" | \"video_off\" | \"video_on\" | \"left_meeting\" | \"share_screen_start\" | \"share_screen_stop\" | \"leave\" | \"dismiss\" | \"page\" | \"settings\" | \"connection\" | \"leave_confirmation\" | \"end_meeting_for_all\" | \"cancel\" | (string & {})) => string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Language"
      },
      "defaultValue": "useLanguage()"
    }
  }; }
  static get states() { return {
    "peers": {},
    "screensharePeers": {},
    "plugins": {},
    "dimensions": {}
  }; }
  static get elementRef() { return "host"; }
  static get watchers() { return [{
      "propName": "meeting",
      "methodName": "meetingChanged"
    }]; }
}
