import { h, proxyCustomElement, HTMLElement, Host } from '@stencil/core/internal/client';
import { R as Render, d as defaultConfig } from './index3.js';
import { d as defaultIconPack } from './default-icon-pack.js';
import { u as useLanguage } from './index2.js';
import { d as defineCustomElement$4 } from './dyte-button2.js';
import { d as defineCustomElement$3 } from './dyte-full-grid2.js';
import { d as defineCustomElement$2 } from './dyte-icon2.js';

const getPeerDimensions = ({ size, container, dimensions, gap, ratio, }) => {
  let [width, height] = dimensions || [container.clientWidth, container.clientHeight];
  width -= gap * 2;
  height -= gap * 2;
  const getArea = (increment) => {
    let w = 0;
    let h = increment * ratio + gap * 2;
    for (let i = 0; i < size; i++) {
      if (w + increment > width) {
        w = 0;
        h += increment * ratio + gap * 2;
      }
      w += increment + gap * 2;
    }
    if (h > height || increment > width)
      return false;
    return increment;
  };
  let max = 0, found = false;
  for (let i = 1; i < 1440; i++) {
    const area = getArea(i);
    if (area === false) {
      max = i - 1;
      found = true;
      break;
    }
  }
  if (!found) {
    max = 1440;
  }
  max -= gap * 2;
  return [max, max * ratio];
};
/**
 * Parses the Aspect Ratio value
 * @param ratio The aspect ratio in the format of `16:9` where `width:height`
 * @returns The parsed value of aspect ratio
 */
const getAspectRatio = (ratio) => {
  const [width, height] = ratio.split(':');
  return Number.parseInt(height) / Number.parseInt(width);
};

const PeersGrid = ({ peers, defaults, host, gap, aspectRatio, containerDimensions, }) => {
  const [width, height] = getPeerDimensions({
    size: peers.length,
    gap,
    dimensions: containerDimensions,
    container: host,
    ratio: getAspectRatio(aspectRatio),
  });
  return peers.map((peer) => (h(Render, { element: "dyte-peer-view", defaults: defaults, props: {
      peer,
      key: peer.id,
      'data-peer': peer.id,
      style: {
        width: `${width}px`,
        height: `${height}px`,
        margin: `${gap}px`,
      },
    }, childProps: { peer }, deepProps: true })));
};

const dyteGridCss = ":host{display:flex;height:100%;width:100%;overflow:hidden;font-family:var(--dyte-font-family, sans-serif);flex-wrap:wrap;align-content:center;align-items:center;justify-content:center;transition:0.4s}dyte-peer-view{aspect-ratio:initial;box-sizing:border-box}";

const DyteGrid$1 = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.__attachShadow();
    /** The aspect ratio of each peer */
    this.aspectRatio = '16:9';
    /** Gap between peers */
    this.gap = 8;
    /** Config object */
    this.config = defaultConfig;
    /** Icon pack */
    this.iconPack = defaultIconPack;
    /** Language */
    this.t = useLanguage();
    this.peers = [];
    this.screensharePeers = [];
    this.plugins = [];
  }
  connectedCallback() {
    this.resizeObserver = new ResizeObserver(() => {
      // update parent dimensions
      const { clientWidth, clientHeight } = this.host;
      this.dimensions = [clientWidth, clientHeight];
    });
    this.resizeObserver.observe(this.host);
    this.meetingChanged(this.meeting);
  }
  disconnectedCallback() {
    var _a;
    (_a = this.resizeObserver) === null || _a === void 0 ? void 0 : _a.disconnect();
    if (this.meeting == null)
      return;
    this.participantJoinedListener &&
      this.meeting.participants.active.removeListener('participantJoined', this.participantJoinedListener);
    this.participantJoinedListener &&
      this.meeting.participants.active.removeListener('participantLeft', this.participantJoinedListener);
    this.screenShareListener &&
      this.meeting.participants.joined.removeListener('screenShareUpdate', this.screenShareListener);
    if (this.viewModeListener != null) {
      this.meeting.participants.removeListener('viewModeChanged', this.viewModeListener);
      this.meeting.participants.removeListener('pageChanged', this.viewModeListener);
    }
    this.screenShareListener &&
      this.meeting.self.removeListener('screenShareUpdate', this.screenShareListener);
  }
  meetingChanged(meeting) {
    if (meeting != null) {
      this.peers = [
        ...meeting.participants.active.toArray(),
        meeting.participants.viewMode === 'ACTIVE_GRID' && meeting.self,
      ];
      this.plugins = [...meeting.plugins.active.toArray()];
      if (this.participantJoinedListener != null)
        return;
      this.participantJoinedListener = (peer) => {
        if (!this.peers.some((p) => p.id === peer.id)) {
          this.peers = [peer, ...this.peers];
        }
      };
      this.participantLeftListener = (peer) => {
        this.peers = this.peers.filter((p) => p.id !== peer.id);
        this.screensharePeers = this.screensharePeers.filter((p) => p.id !== peer.id);
      };
      this.screenShareListener = (peer) => {
        if (peer.screenShareEnabled && peer.screenShareTracks.video != null) {
          this.addScreenSharePeer(peer);
        }
        else {
          this.removeScreenSharePeer(peer.id);
        }
      };
      this.selfScreenShareListener = ({ screenShareEnabled, screenShareTracks }) => {
        const peer = meeting.self;
        if (screenShareEnabled && screenShareTracks.video != null) {
          this.addScreenSharePeer(peer);
        }
        else {
          this.removeScreenSharePeer(peer.id);
        }
      };
      this.viewModeListener = ({ viewMode, currentPage, pageCount }) => {
        if (viewMode === 'ACTIVE_GRID') {
          if (!this.peers.some((p) => p.id === meeting.self.id)) {
            // add self if it doesn't exist
            this.peers = [...this.peers, this.meeting.self];
          }
        }
        else if (currentPage !== pageCount) {
          // remove self peer only if the current page is not the last page
          this.peers = this.peers.filter((p) => p.id !== this.meeting.self.id);
        }
      };
      this.pageChangedListener = ({ currentPage, pageCount, viewMode }) => {
        if (viewMode === 'PAGINATED') {
          if (currentPage === pageCount) {
            if (!this.peers.some((p) => p.id === meeting.self.id)) {
              // add self peer if it is the last page and it already isn't added
              this.peers = [...this.peers, meeting.self];
            }
          }
        }
      };
      this.pluginUpdateListener = (plugin, { active }) => {
        if (active) {
          if (!this.plugins.some((p) => p.id === plugin.id)) {
            this.plugins = [...this.plugins, plugin];
          }
        }
        else {
          this.plugins = this.plugins.filter((p) => p.id !== plugin.id);
        }
      };
      {
        const activeParticipants = meeting.participants.active.toArray();
        for (const peer of activeParticipants) {
          this.screenShareListener(peer);
        }
      }
      meeting.participants.active.addListener('participantJoined', this.participantJoinedListener);
      meeting.participants.active.addListener('participantLeft', this.participantLeftListener);
      meeting.participants.active.addListener('screenShareUpdate', this.screenShareListener);
      meeting.participants.addListener('viewModeChanged', this.viewModeListener);
      meeting.participants.addListener('pageChanged', this.pageChangedListener);
      meeting.self.addListener('screenShareUpdate', this.selfScreenShareListener);
      meeting.plugins.all.addListener('stateUpdate', this.pluginUpdateListener);
    }
  }
  addScreenSharePeer(peer) {
    if (!this.screensharePeers.some((p) => p.id === peer.id)) {
      this.screensharePeers = [...this.screensharePeers, peer];
    }
  }
  removeScreenSharePeer(id) {
    this.screensharePeers = this.screensharePeers.filter((p) => p.id !== id);
  }
  render() {
    const isPeersOnly = this.screensharePeers.length + this.plugins.length === 0;
    const defaults = {
      meeting: this.meeting,
      size: this.size,
      states: this.states,
      config: this.config,
      iconPack: this.iconPack,
      t: this.t,
    };
    const common = {
      peers: this.peers,
      defaults,
      gap: this.gap,
      aspectRatio: this.aspectRatio,
      containerDimensions: this.dimensions,
      host: this.host,
    };
    return (h(Host, null, isPeersOnly && (h(PeersGrid, Object.assign({}, common, { host: this.host, containerDimensions: this.dimensions }))), !isPeersOnly && (h("dyte-full-grid", Object.assign({}, defaults, common, { screenSharePeers: this.screensharePeers, plugins: this.plugins }))), h(Render, { element: "dyte-grid", defaults: defaults, onlyChildren: true })));
  }
  get host() { return this; }
  static get watchers() { return {
    "meeting": ["meetingChanged"]
  }; }
  static get style() { return dyteGridCss; }
}, [1, "dyte-grid", {
    "aspectRatio": [513, "aspect-ratio"],
    "meeting": [16],
    "gap": [514],
    "size": [513],
    "states": [16],
    "config": [16],
    "iconPack": [16],
    "t": [16],
    "peers": [32],
    "screensharePeers": [32],
    "plugins": [32],
    "dimensions": [32]
  }]);
function defineCustomElement$1() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["dyte-grid", "dyte-button", "dyte-full-grid", "dyte-icon"];
  components.forEach(tagName => { switch (tagName) {
    case "dyte-grid":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, DyteGrid$1);
      }
      break;
    case "dyte-button":
      if (!customElements.get(tagName)) {
        defineCustomElement$4();
      }
      break;
    case "dyte-full-grid":
      if (!customElements.get(tagName)) {
        defineCustomElement$3();
      }
      break;
    case "dyte-icon":
      if (!customElements.get(tagName)) {
        defineCustomElement$2();
      }
      break;
  } });
}

const DyteGrid = DyteGrid$1;
const defineCustomElement = defineCustomElement$1;

export { DyteGrid, defineCustomElement };
